# -*- coding: utf-8 -*-

import re
import time
import requests
from .parser import routes
from .article_list import ArticleList

BASE_URL = "https://www.ptt.cc"
URL_FORMAT = "/bbs/{board}{path}"
REQUEST_FAILED_MESSAGE = "HTTP request failed! status: {status}, url: {url}"
UNKNOWN_PAGE_MESSAGE = "Unknown page {url}"
IS_URL = re.compile("^https?:\/\/")
PATH_WITH_BOARD_NAME = re.compile("^\/bbs\/(?:.+)\/")


class Board:
    def __init__(self, name, verify=True):
        self.name = name
        self.verify = verify
        self.cookies = dict(over18="1")

    def articles(self):
        return ArticleList(self)

    def get_data(self, url):
        url = self.get_url(url)
        parse = routes(url)

        while True:
            r = requests.get(url, verify=self.verify, cookies=self.cookies)

            if parse is None:
                raise Exception(UNKNOWN_PAGE_MESSAGE.format(url=url))

            if r.status_code == 503:
                time.sleep(2)
                continue

            if r.status_code is not 200:
                msg = REQUEST_FAILED_MESSAGE.format(url=url, status=r.status_code)
                raise Exception(msg)

            break

        return parse(r.text)

    def get_url(self, path):
        if IS_URL.search(path):
            return path

        if not PATH_WITH_BOARD_NAME.search(path):
            path = URL_FORMAT.format(board=self.name, path=path)

        return BASE_URL + path



# -*- coding: utf-8 -*-

"""
    處理文章頁面資料
"""

import re
from functools import reduce
from bs4 import BeautifulSoup

__all__ = ("parse")

IS_REPLY = re.compile("^\s*RE:")
PARSE_TITLE = re.compile("(?:\[(.*?)\])?\s*(.*)$")

META_NAMES = {
    u"作者": "author",
    u"看板": "board",
    u"標題": "title",
    u"時間": "time",
}


def parse(html):
    data = {}
    soup = BeautifulSoup(html)
    main = soup.find(id="main-content")

    for node in main.contents:
        parse_node(node, data)

    data["content"] = data.get("content", "").strip()

    return data


def parse_node(node, context):
    parse_meta(node, context)\
        or parse_comment(node, context)\
        or parse_content(node, context)


def parse_meta(node, context):
    if is_element(node) and is_meta(node):
        meta = context.get("meta", {})

        key = node.select(".article-meta-tag")
        val = node.select(".article-meta-value")

        if len(val) is 0 or len(key) is 0:
            return True

        key = META_NAMES.get(key[0].text.strip(), "unknown")
        val = val[0].text.strip()
        meta[key] = val

        if key is "title":
            match = PARSE_TITLE.match(meta["title"])
            article_type = match.group(1)
            title = match.group(2)
            meta["type"] = article_type
            meta["title"] = title
            meta["re"] = True if IS_REPLY.search(title) else False

        context["meta"] = meta

        return True


def parse_comment(node, context):
    if is_element(node) and is_comment(node):
        tag = node.select(".push-tag")
        user = node.select(".push-userid")
        content = node.select(".push-content")
        time = node.select(".push-ipdatetime")

        if len(tag) and len(user) and len(time):
            comments = context.get("comments", [])
            comments.append({
                "tag": tag[0].text.strip(),
                "user": user[0].text.strip(),
                "time": time[0].text.strip(),
                "content": get_comment_content(content[0]),
            })
            context["comments"] = comments

        return True


def parse_content(node, context):
    context["content"] = context.get("content", "") + get_text(node)


def get_comment_content(comment_root):
    content = ""

    for node in comment_root.contents:
        content += get_text(node)

    return content.strip()


def get_text(node):
    if is_link(node):
        return node.get("href", "")
    elif is_image(node):
        return node.get("src", "")
    return node.text if is_element(node) else node


def is_element(node):
    return node.name


def is_link(node):
    return node.name is "a"


def is_image(node):
    return node.name is "img"


def is_comment(node):
    return has_class(node, "push")


def is_meta(node):
    return has_class(node, "article-metaline", "article-metaline-right")


def has_class(node, *args):
    expect = list(args)
    expect.insert(0, False)
    names = node.get("class", [])
    return reduce(lambda success, name: success or name in names, expect)



# -*- coding: utf-8 -*-

import re
from dateutil.parser import parse as parseTime

PARSE_ID = re.compile("((?:\w+\.){3}\w+)")


class Article:
    def __init__(self, path, board):
        self.path = path
        self.board = board
        self.data = {}

    def readable(self):
        return self.data.get("meta")

    @property
    def id(self):
        match = PARSE_ID.search(self.path)
        return match and match.group(1)

    @property
    def url(self):
        return self.board.get_url(self.path)

    @property
    def title(self):
        return self.get_meta_data("title")

    @property
    def author(self):
        return self.get_meta_data("author")

    @property
    def time(self):
        time = self.get_meta_data("time")
        return parseTime(time)

    @property
    def type(self):
        return self.get_meta_data("type")

    @property
    def reply(self):
        return self.get_meta_data("re")

    @property
    def content(self):
        return self.data.get("content")

    def get_meta_data(self, key, defval=None):
        meta = self.data.get("meta", {})
        return meta.get(key, defval)

    def fetch(self):
        self.data = self.board.get_data(self.path)



# -*- coding: utf-8 -*-

from bs4 import BeautifulSoup

__all__ = ("parse")


def parse(html):
    soup = BeautifulSoup(html)
    url = parse_prev_page_url(soup)

    if url is None:
        return None

    return {
        "prev_page_url": url,
        "article_list": parse_article_list(soup),
    }


def parse_prev_page_url(soup):
    links = soup.select(".action-bar .pull-right .btn")
    url = None

    try:
        url = links[1].get("href")
    except:
        pass

    return url


def parse_article_list(soup):
    links = soup.select(".r-ent a")
    links = [link.get('href') for link in links]
    return links



# -*- coding: utf-8 -*-

from .article import Article

NO_MORE_DATA_MESSAGE = "No more data!"


class ArticleList:
    def __init__(self, board):
        self.board = board
        self.reset()

    def __iter__(self):
        return self

    def __next__(self):
        return self.next_article()

    def next(self):
        return self.next_article()

    def next_article(self):
        if self.buffer_is_empty():
            self.next_page()
            if self.at_last_page:
                raise StopIteration

        article_url = self.buffer[self.buffer_cursor]
        self.buffer_cursor += 1

        return Article(article_url, self.board)

    def next_page(self):
        if self.at_last_page:
            raise Exception(NO_MORE_DATA_MESSAGE)

        count = 0
        data = None
        page_url = self.page_url

        while count is 0:
            data = self.board.get_data(page_url)

            if data is None:
                self.at_last_page = True
                self.buffer = None
                self.buffer_cursor = 0
                self.buffer_lastindex = -1
                return

            count = len(data["article_list"])
            page_url = data["prev_page_url"]

        self.buffer_cursor = 0
        self.buffer = data["article_list"]
        self.buffer_lastindex = count - 1
        self.page_url = page_url

    def buffer_is_empty(self):
        return self.buffer_cursor > self.buffer_lastindex

    def reset(self):
        self.page_url = "/index.html"
        self.at_last_page = False
        self.buffer = None
        self.buffer_cursor = 0
        self.buffer_lastindex = -1



try:
    import multiprocessing
except ImportError:
    pass

import re
from setuptools import setup

with open('ptt_crawler/__init__.py') as f:
    m = re.findall(r'__version__\s*=\s*"(.*)"', f.read())
    version = m[0]

setup(
    name='ptt_crawler',
    version=version,
    url='https://github.com/youmeb-lab/ptt-crawler',
    license='MIT',
    author='Po-Ying Chen, Jamie Yang',
    author_email='poying.me@gmail.com',
    packages=[
        'ptt_crawler',
        'ptt_crawler.parser',
    ],
    platforms='any',
    install_requires=[
        'requests>=2.5.1',
        'beautifulsoup4',
    ],
    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Developers',
        'Natural Language :: Chinese (Traditional)',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ]
)



# -*- coding: utf-8 -*-
# flake8: noqa

import re
from .list import parse as parse_list
from .article import parse as parse_article

ARTICLE_RE = re.compile("\/bbs\/[-_\w]+\/(?:\w+\.){4}html$")
LIST_RE = re.compile("\/index\d*\.html$")


def routes(url):
    return article_page(url) \
        or list_page(url)


def article_page(url):
    if ARTICLE_RE.search(url):
        return parse_article


def list_page(url):
    if LIST_RE.search(url):
        return parse_list
